function [Wm, Hm, sigmam, nlp] = BayesNMF_Vol(X, N, M, varargin)

% VOLNMF Non-negative matrix factorization with volume contraints
%
% Usage
%   [Wm,Hm,sm] = volnmf(X,N,M,[options])
%
% Input
%   X           Data matrix (I x J)
%   N           Number of components
%   M           Number of Gibbs samples to compute
%   options
%     reg       Regularization type ('pp', 'vol', or 'dist')
%     gamma     Volume prior
%     theta     Prior for sigma
%     k         Prior for sigma
%
%     W         Initial value for W (I x N)
%     H         Initial value for H (N x J)
%     sigma     Initial value for noise variance (sigma^2)
%
%     chains    Number of chains to run (default 1)
%     skip      Number of initial samples to skip (default 100)
%     stride    Return every n'th sample (default 1)
%
%     progress  Display percentage completed and estimated time left (bool)
%
% Output
%   Wm          Samples of W (I x N x M)
%   Hm          Samples of H (N x J x M)
%   sm          Samples of sigma (M x 1)
%
% Authors
%   Mikkel N. Schmidt,
%   DTU Informatics, Technical University of Denmark.
%   University of Cambridge, Department of engineering, UK.
%
%   Morten Arngren
%   DTU Informatics, Technical University of Denmark.
%   FOSS Analytical A/S, Hilleroed, Denmark
%
% Copyright 2009-2010 Mikkel N. Schmidt, ms@it.dk, www.mikkelschmidt.dk
%                     Morten Arngren, ma@imm.dtu.dk, phd.arngren.com
%
% V1.0 March 2010   : Initial issue.
%

[I,J] = size(X);

opts     = mgetopt(varargin);
reg      = mgetopt(opts, 'reg', 'pp');
gamma    = mgetopt(opts, 'gamma', 0);
theta    = mgetopt(opts, 'theta', 0);
k        = mgetopt(opts, 'k', 0);
W0       = mgetopt(opts, 'W', rand(I,N));
H0       = mgetopt(opts, 'H', rand(N,J)); H0 = H0./repmat(sum(H0,1),N,1);
W_true   = mgetopt(opts, 'W_true', 0);
sigma0   = mgetopt(opts, 'sigma', 1);
chains   = mgetopt(opts, 'chains', 1);
stride   = mgetopt(opts, 'stride', 1);
skip     = mgetopt(opts, 'skip', 0);
rep      = mgetopt(opts, 'rep', 1);
progress = mgetopt(opts, 'progress', false');
doPlot   = mgetopt(opts, 'doPlot', 0);

nlp      = zeros(M*chains,1);
Wm       = zeros(I,N,M*chains);
Hm       = zeros(N,J,M*chains);
sigmam   = zeros(M*chains,1);

% Compress W to K-1 dimensions based on data X
meanX   = mean(X,2);
X_      = X - meanX(:,ones(size(X,2),1));
[U D]   = svd(X_*X_'/size(X,2));
U       = U(:,1:N-1);
UX      = U'*X;           % Only for plotting during iterations.
UW_true = U'*W_true;

x = sum(X(:).^2)/2;
if progress, mprogress; ic = 0; IC = chains*(skip+(M-1)*max(stride,1)); end;
for r = 1:chains
  W = W0;
  H = H0;
  sigma = sigma0;
  for m = 1:M
    for t = 1:max(skip*(m==1)+stride*(m>1),1)
      C = H*H';
      D = X*H';
      for irep = 1:rep
        for n = 1:N
          nn = [1:n-1 n+1:N];
          switch reg
            case 'pp'
              G = W(:,nn)'*W(:,nn);
              adjG = adj(G);
              detG = det(G);
            case 'vol'
              np = mod(n,N)+1;
              nnpt = mod((1:N-2)+n,N)+1;
              Gt = bsxfun(@minus,W(:,nnpt),W(:,np))'*bsxfun(@minus,W(:,nnpt),W(:,np));
              adjGt = adj(Gt);
              detGt = det(Gt);
            case 'dist'
          end
          for i = 1:I
            ni = [1:i-1 i+1:I];
            m1 = (D(i,n)-W(i,nn)*C(nn,n))/C(n,n);
            iv1 = C(n,n)/sigma;
            switch reg
              case 'pp'
                b  = W(i,nn)';
                c  = W(ni,nn)'*W(ni,n);
                t2 = gamma*(detG-b'*adjG*b);
                t1 = -2*gamma*b'*adjG*c;
              case 'vol'
                b  = W(i,nnpt)'-W(i,np);
                c  = (bsxfun(@minus,W(ni,nnpt),W(ni,np)))'*(W(ni,n)-W(ni,np));
                t2 = gamma*(detGt-b'*adjGt*b);
                t1 = -2*W(i,np)*t2-2*gamma*b'*adjGt*c;
              case 'dist'
                t2 = gamma*(N-1)/N;
                t1 = -2*gamma/N*sum(W(i,nn));
              otherwise
                error('Wrong regularization specified.');
            end
            iv2    = t2;
            iv2m2  = -t1/2;
            v      = 1/(iv1+iv2);
            mu     = v*(iv1*m1+iv2m2);
            s      = sqrt(-2*(log(rand)-.5*W(i,n).^2));
            ll     = max(-s, -mu/sqrt(v));
            uu     = s;
            W(i,n) = (rand.*(uu-ll)+ll)*sqrt(v)+mu;
          end
        end
      end
      sigma = 1/gamrnd((I*J)/2+1+k, ...
        1/(x+theta+sum(sum(W.*(W*C-2*D)))/2));
      nlp(m+(r-1)*M) = ...
        -(x+sum(sum(W.*(W*(C)-2*D)))/(2*sigma))...
        -gamma*det(W'*W) ...
        -(x+theta+sum(sum(W.*(W*(C)-2*D)))/(2*sigma))-log(sigma)*((I*J)/2+1+k+1);
      E  = W'*W;
      F  = W'*X;
      Mu = E\F; S = E\(sigma*eye(N));
      H  = randcg(Mu,S,eye(N),zeros(N,1),ones(1,N),1,H,rep);
      
      if progress, ic = ic+1; mprogress(ic/IC); end;
    end
    Wm(:,:,m+(r-1)*M) = W;
    Hm(:,:,m+(r-1)*M) = H;
    sigmam(m+(r-1)*M) = sigma;
    
    if (rem(m,doPlot)==0),
      if (rem(m,doPlot)==0),
        UW  = U(:,1:N-1)'*W;
        UWH = U(:,1:N-1)'*W*H;
        
        figure(100); clf; hold on;
        id = 1;
        for ii = 1:N-2
          for jj = ii+1:N-1
            subplot(N-2,N-2,id); grid on; hold on;
            plot(UX(ii,:),  UX(jj,:),  'or');
            plot(UWH(ii,:), UWH(jj,:), '.y');
            if W_true, plot(UW_true(1,:), UW_true(2,:), 'kx', 'MarkerSize', 15, 'LineWidth', 3); end
            UWpi = perms(UW(ii,:));
            UWpj = perms(UW(jj,:));
            plot(UWpi, UWpj, '-b', 'LineWidth', 2);
            title([num2str(jj)]); ylabel([num2str(ii)]); hold off;
            id = id + 1;
          end
          id = id + ii;
        end
      end
    end
    
    
  end
end

%--------------------------------------------------------------------------
function out = mgetopt(varargin)
% MGETOPT Parser for optional arguments
%
% Usage
%   Get a parameter structure from 'varargin'
%     opts = mgetopt(varargin);
%
%   Get and parse a parameter:
%     var = mgetopt(opts, varname, default);
%        opts:    parameter structure
%        varname: name of variable
%        default: default value if variable is not set
%
%     var = mgetopt(opts, varname, default, command, argument);
%        command, argument:
%          String in set:
%          'instrset', {'str1', 'str2', ... }
%
% Example
%    function y = myfun(x, varargin)
%    ...
%    opts = mgetopt(varargin);
%    parm1 = mgetopt(opts, 'parm1', 0)
%    ...

% Copyright 2007 Mikkel N. Schmidt, ms@it.dk, www.mikkelschmidt.dk

if nargin==1
  if isempty(varargin{1})
    out = struct;
  elseif isstruct(varargin{1})
    out = varargin{1}{:};
  elseif isstruct(varargin{1}{1})
    out = varargin{1}{1};
  else
    out = cell2struct(varargin{1}(2:2:end),varargin{1}(1:2:end),2);
  end
elseif nargin>=3
  opts    = varargin{1};
  varname = varargin{2};
  default = varargin{3};
  validation = varargin(4:end);
  if isfield(opts, varname)
    out = opts.(varname);
  else
    out = default;
  end
  
  for narg = 1:2:length(validation)
    cmd = validation{narg};
    arg = validation{narg+1};
    switch cmd
      case 'instrset',
        if ~any(strcmp(arg, out))
          fprintf(['Wrong argument %sigma = ''%sigma'' - ', ...
            'Using default : %sigma = ''%sigma''\n'], ...
            varname, out, varname, default);
          out = default;
        end
      case 'dim'
        if ~all(size(out)==arg)
          fprintf(['Wrong argument dimension: %sigma - ', ...
            'Using default.\n'], ...
            varname);
          out = default;
        end
      otherwise,
        error('Wrong option: %sigma.', cmd);
    end
  end
end


function x = randcg(mx, Sxx, A, b, Aeq, beq, x0, T)
% RANDCG Random numbers from constrained Gaussian density
%   p(x) = const * N(mx,Sxx), s.t. A*x-b<0, Aeq*x-beq=0
%
% Usage
%   x = randcg(mx, Sxx, A, b, Aeq, beq) draws a random vector from the
%   constrained multivariate Gaussian density.
%
% Input
%   mx          Mean vector
%   Sxx         Covariance matrix
%   A, b        Matrix and vector specifying inequality constraints
%   Aeq, beq    Matrix and vector specifying equality constraints

% Copyright 2009-2010 Mikkel N. Schmidt, ms@it.dk, www.mikkelschmidt.dk
%                     Morten Arngren, ma@imm.dtu.dk, phd.arngren.com

% Dimensionality of data
N = size(mx,1);

% Orthogonal basis of equality constraints + new origin
if isempty(Aeq) % No equality constraints: use standard basis
  P  = zeros(0,N);
  K  = eye(N);
  cx = zeros(N,1);
else
  [U,S] = svd(Aeq');
  [m,n] = size(Aeq);
  if m>1, s=diag(S); elseif m==1, s=S(1); else s=0; end
  tol   = max(m,n)*max(s)*eps;
  r     = sum(s>tol);
  P     = U(:,1:r)';
  K     = U(:,r+1:end)';
  cx    = Aeq\beq;
end
if isempty(A) % No inequality constraints
  A = zeros(0,N);
  b = zeros(0,1);
end

% Dimensionality of space that satisfies eq. constraints
M  = N-size(P,1);

W  = K*(eye(N)-Sxx*P'*((P*Sxx*P')\P));
my = W*bsxfun(@minus,mx,cx);
L  = chol(W*Sxx*K');

% Start point
w = L'\bsxfun(@minus, K*bsxfun(@minus, x0, cx), my);

% Precomputations for bounds
E = A*K'*L';
e = bsxfun(@minus, b, A*bsxfun(@plus, K'*my, cx));

% Loop over Gibbs sweeps
for t = 1:T
  % Loop over individual elements
  for m = 1:M
    % All indices except m
    nm = [1:m-1 m+1:M];
    
    % Compute lower and upper bound
    n  = e-E(:,nm)*w(nm,:); %-EW
    d  = E(:,m);
    lb = max(bsxfun(@rdivide,n(d>0,:),d(d>0,:)),[],1);
    if isempty(lb), lb = -inf; end
    ub = min(bsxfun(@rdivide,n(d<0,:),d(d<0,:)),[],1);
    if isempty(ub), ub = inf; end;
    
    % Draw from truncated Gaussian density
    w(m,:) = randstgs(lb, ub, w(m,:));
  end
end

% Final result mapped back to the original space
x = bsxfun(@plus, K'*bsxfun(@plus, L'*w, my), cx);



function x = randstgs(l, u, x)
% RANDTG Random numbers from standard truncated Gaussian density
%   p(x) = const * exp(-x^2/2) *I(l < x < u) using slice sampling
%
% Usage
%   x = randtg(l, u, x0) returns an array of random numbers chosen
%     from the truncated Gaussian density. The size of x is the maximum
%     common size of the parameters.
%
% Input
%   l, u         Truncation, l < x < u

% Copyright 2009-2010 Mikkel N. Schmidt, ms@it.dk, www.mikkelschmidt.dk
%                     Morten Arngren, ma@imm.dtu.dk, phd.arngren.com

sz = size(l);
z  = bsxfun(@plus, -.5*x.^2, log(rand(sz)));
s  = sqrt(-2*z);
ll = bsxfun(@max, -s, l);
uu = bsxfun(@min, s, u);
x  = rand(sz).*(uu-ll)+ll;




